"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const iOSProxyServices = require("../../common/mobile/ios/device/ios-proxy-services");
class IOSSocketRequestExecutor {
    constructor($errors, $injector, $iOSNotification, $iOSNotificationService, $logger) {
        this.$errors = $errors;
        this.$injector = $injector;
        this.$iOSNotification = $iOSNotification;
        this.$iOSNotificationService = $iOSNotificationService;
        this.$logger = $logger;
    }
    executeAttachRequest(device, timeout, projectId) {
        return __awaiter(this, void 0, void 0, function* () {
            let npc = new iOSProxyServices.NotificationProxyClient(device, this.$injector);
            let data = [this.$iOSNotification.getAlreadyConnected(projectId), this.$iOSNotification.getReadyForAttach(projectId), this.$iOSNotification.getAttachAvailable(projectId)]
                .map((notification) => this.$iOSNotificationService.awaitNotification(npc, notification, timeout)), alreadyConnected = data[0], readyForAttach = data[1], attachAvailable = data[2];
            npc.postNotificationAndAttachForData(this.$iOSNotification.getAttachAvailabilityQuery(projectId));
            let receivedNotification;
            try {
                receivedNotification = yield Promise.race([alreadyConnected, readyForAttach, attachAvailable]);
            }
            catch (e) {
                this.$errors.failWithoutHelp(`The application ${projectId} does not appear to be running on ${device.deviceInfo.displayName} or is not built with debugging enabled.`);
            }
            switch (receivedNotification) {
                case this.$iOSNotification.getAlreadyConnected(projectId):
                    this.$errors.failWithoutHelp("A client is already connected.");
                    break;
                case this.$iOSNotification.getAttachAvailable(projectId):
                    yield this.executeAttachAvailable(npc, timeout, projectId);
                    break;
                case this.$iOSNotification.getReadyForAttach(projectId):
                    break;
            }
        });
    }
    executeLaunchRequest(device, timeout, readyForAttachTimeout, projectId, shouldBreak) {
        return __awaiter(this, void 0, void 0, function* () {
            let npc = new iOSProxyServices.NotificationProxyClient(device, this.$injector);
            try {
                yield this.$iOSNotificationService.awaitNotification(npc, this.$iOSNotification.getAppLaunching(projectId), timeout);
                process.nextTick(() => {
                    if (shouldBreak) {
                        npc.postNotificationAndAttachForData(this.$iOSNotification.getWaitForDebug(projectId));
                    }
                    npc.postNotificationAndAttachForData(this.$iOSNotification.getAttachRequest(projectId));
                });
                yield this.$iOSNotificationService.awaitNotification(npc, this.$iOSNotification.getReadyForAttach(projectId), readyForAttachTimeout);
            }
            catch (e) {
                this.$logger.trace(`Timeout error: ${e}`);
                this.$errors.failWithoutHelp("Timeout waiting for response from NativeScript runtime.");
            }
        });
    }
    executeAttachAvailable(npc, timeout, projectId) {
        return __awaiter(this, void 0, void 0, function* () {
            process.nextTick(() => npc.postNotificationAndAttachForData(this.$iOSNotification.getAttachRequest(projectId)));
            try {
                yield this.$iOSNotificationService.awaitNotification(npc, this.$iOSNotification.getReadyForAttach(projectId), timeout);
            }
            catch (e) {
                this.$errors.failWithoutHelp(`The application ${projectId} timed out when performing the socket handshake.`);
            }
        });
    }
}
exports.IOSSocketRequestExecutor = IOSSocketRequestExecutor;
$injector.register("iOSSocketRequestExecutor", IOSSocketRequestExecutor);
